import User from "../model/user.js";
import SibApiV3Sdk from "sib-api-v3-sdk";
import crypto from "crypto";
import partnerotpSchema from "../model/Partnersotp.js";
// import User from "../model/user.js";
import City from '../model/FranchiseCity.model.js'

// ✅ Function to Generate a Unique Reference Number
const generateReferenceNumber = () => {
    return "FR-" + crypto.randomBytes(3).toString("hex").toUpperCase();
};

// ✅ Send Email Using Brevo API
const sendEmail = async (to, subject, params, templateId) => {
    try {
        if (!process.env.BREVO_API_KEY || !process.env.BREVO_EMAIL) {
            throw new Error("❌ [Brevo] Missing API Key or Sender Email in .env file!");
        }

        const client = SibApiV3Sdk.ApiClient.instance;
        const apiKey = client.authentications["api-key"];
        apiKey.apiKey = process.env.BREVO_API_KEY;

        const apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        const emailData = {
            sender: { email: process.env.BREVO_EMAIL },
            to: [{ email: to }],
            templateId: templateId, // Dynamic Template ID passed to function
            params: params,
        };

        await apiInstance.sendTransacEmail(emailData);
        console.log(`✅ [Brevo] Email sent to: ${to}`);
    } catch (error) {
        console.error("❌ [Brevo] Email Error:", error.response?.body || error);
    }
};

// ✅ Register a Franchise
export const registerFranchise = async (req, res) => {
    try {
        const {  ownerName, email, phone, address, city } = req.body;
        let city_id;

        // Restrict city from being 'Cuttack' or 'Bhubaneswar' (case insensitive)
        const restrictedCities = ["Cuttack", "Bhubaneswar"];
        if (restrictedCities.map(c => c.toLowerCase()).includes(city.trim().toLowerCase())) {
            return res.status(400).json({ message: `Franchises cannot be registered in ${city}.` });
        }

        // Check if email, phone already exists
        const existingFranchise = await User.findOne({ 
            $or: [{ email }, { phone }] 
        });

        if (existingFranchise) {
            if (existingFranchise.email === email) {
                return res.status(400).json({ message: "Email already exists!" });
            }
            if (existingFranchise.phone === phone) {
                return res.status(400).json({ message: "Phone number already exists!" });
            }
        }

        // Generate Reference Number
        const referenceNumber = generateReferenceNumber();

        const user_type="full_franchise";
        const fetch_password=await partnerotpSchema.find({email:email});
        if(!fetch_password){
            return res.status(404).json({message:"User not found"})
        }
        const password=fetch_password[0].password;
        const reference_number=referenceNumber

        //write the logic here to save the city in the database with id and all
        const create_city=await City.create(
            {
                cityName:city,
                
            }
        )
        if(create_city){
             city_id=create_city._id
        }


        // Create a new Franchise entry
        const newFranchise = new User({ ownerName, email, phone, address, city, reference_number,user_type,password,city_id });
        console.log("the object to save in the database is",newFranchise);
        await newFranchise.save();

        // ✅ Send Email to User after successful registration
        const userTemplateId = 43; // User Email Template ID
        await sendEmail(email, "Franchise Registration Successful", { ownerName, referenceNumber, currentYear: new Date().getFullYear() }, userTemplateId);

        // ✅ Send Email to Admins after successful registration
        const adminTemplateId = 44; // Admin Email Template ID
        const adminEmails = [process.env.ADMIN_EMAIL_1, process.env.ADMIN_EMAIL_2]; // Use two admin emails
        for (let adminEmail of adminEmails) {
            await sendEmail(adminEmail, "New Franchise Registration", { ownerName, email, phone, referenceNumber, currentYear: new Date().getFullYear() }, adminTemplateId);
        }

        // Return successful response
        res.status(201).json({ message: "Franchise registered successfully!", referenceNumber, User: newFranchise });
    } catch (error) {
        res.status(500).json({ message: "Server Error", error: error.message });
    }
};
