import User from "../model/user.js";
import jwt from "jsonwebtoken";
import bcrypt from "bcrypt";
// import nodemailer from "nodemailer";
import dotenv from "dotenv";
import { generateToken } from "../utils/auth.js";
import crypto from "crypto";
// import transporter from "../config/emailConfig.js";
// import axios from "axios"
import SibApiV3Sdk from 'sib-api-v3-sdk';
import partnerotpSchema from "../model/Partnersotp.js";



dotenv.config();





// ✅ Function to Generate OTP
const generateOTP = () => Math.floor(100000 + Math.random() * 900000).toString();

// ✅ Send OTP Email Using Brevo API
const sendOTPEmail = async (email, otp, templateId) => {
    console.log("🔹 [Brevo] Sending OTP to:", email);

    try {
        // ✅ Ensure API Key is Set
        if (!process.env.BREVO_API_KEY) {
            throw new Error("❌ [Brevo] Missing API Key! Check .env file.");
        }

        // ✅ Initialize Brevo API Client
        const defaultClient = SibApiV3Sdk.ApiClient.instance;
        const apiKey = defaultClient.authentications["api-key"];
        apiKey.apiKey = process.env.BREVO_API_KEY;

        // ✅ Create API Instance
        const apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();

        // ✅ Ensure Brevo Email is Set
        if (!process.env.BREVO_EMAIL) {
            throw new Error("❌ [Brevo] Missing Sender Email in .env file!");
        }

        // ✅ Email Data
        const emailData = {
            sender: { email: process.env.BREVO_EMAIL }, // Must be verified in Brevo
            to: [{ email: email }],
            templateId: templateId,
            params: { otp: otp, USER_EMAIL: email },
        };

        // ✅ Send Email
        const response = await apiInstance.sendTransacEmail(emailData);
        console.log("✅ [Brevo] Email sent:", response);

    } catch (error) {
        console.error("❌ [Brevo] Email Error:", error.response?.body || error);
        throw new Error("Failed to send OTP email.");
    }
};

// ✅ Signup (Generate & Send OTP)
export const sendotp = async (req, res) => {
    try {
      console.log("🔹 [Signup] Request received:", req.body);
  
      const { email } = req.body;
      if (!email) {
        return res.status(400).json({ error: "Email is required" });
      }
  
      console.log("🔹 [Signup] Checking user...");
      let user = await partnerotpSchema.findOne({ email });
  
      if (!user) {
        user = new partnerotpSchema({ email });
      }
  
      // ✅ Generate OTP
      let otp = generateOTP();
      console.log("🔹 [Signup] OTP:", otp);
  
      user.otp = otp;
      user.otpExpiry = new Date(Date.now() + 10 * 60 * 1000); // 10 mins
      await user.save();
  
      // ✅ Send OTP via Brevo Template ID: 41
      try {
        await sendOTPEmail(email, otp, 41); // Send OTP email
        console.log('OTP sent successfully to:', email);
      } catch (emailError) {
        console.error('❌ [Signup] Error sending OTP email:', emailError.message);
        return res.status(500).json({ error: 'Failed to send OTP email.' });
      }
  
      return res.status(200).json({ message: "OTP sent successfully" });
  
    } catch (error) {
      console.error("❌ [Signup] Server Error:", error);
      return res.status(500).json({ error: error.message || "Internal Server Error" });
    }
  };
  



// ✅ Verify OTP
export const verifyOTP = async (req, res) => {
    try {
        const { email, otp } = req.body;
        if (!email || !otp) return res.status(400).json({ error: "Email and OTP are required" });

        const user = await partnerotpSchema.findOne({ email });

        if (!user || user.otp !== otp || new Date() > user.otpExpiry) {
            return res.status(400).json({ error: "Invalid or Expired OTP" });
        }

        // ✅ Generate Access Token
        const accessToken = jwt.sign({ email: user.email }, process.env.JWT_SECRET, { expiresIn: "1h" });

        return res.json({ message: "OTP Verified", accessToken });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
};





const sendPasswordEmail = async (email, password) => {
    console.log("🔹 [Brevo] Sending Password Update Email to:", email);

    try {
        // ✅ Ensure API Key is Set
        if (!process.env.BREVO_API_KEY) {
            throw new Error("❌ [Brevo] Missing API Key! Check .env file.");
        }

        // ✅ Initialize Brevo API Client
        const brevoClient = SibApiV3Sdk.ApiClient.instance;
        const apiKey = brevoClient.authentications["api-key"];
        apiKey.apiKey = process.env.BREVO_API_KEY;

        // ✅ Create API Instance
        const apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();

        // ✅ Email Data
        const emailData = {
            sender: { name: "RecyclePay", email: "partners.recyclepay@ceibagreen.com" }, // Hardcoded sender email
            to: [{ email }],
            templateId: 42,
            params: { password },
        };

        // ✅ Send Email
        const response = await apiInstance.sendTransacEmail(emailData);
        console.log("✅ [Brevo] Password Update Email Sent:", response);

    } catch (error) {
        console.error("❌ [Brevo] Email Error:", error.response?.body || error);
        throw new Error("Failed to send password update email.");
    }
};



export const setPassword = async (req, res) => {
    try {
        console.log("Request Body:", req.body);

        const { newPassword, confirmPassword } = req.body;
        console.log("the password and the confirm password coming from the frontend is",newPassword,confirmPassword);

        if (!newPassword || !confirmPassword) {
            return res.status(400).json({ error: "Both password fields are required" });
        }

        if (newPassword !== confirmPassword) {
            return res.status(400).json({ error: "Passwords do not match" });
        }

        const email = req.user.email;
        const hashedPassword = await bcrypt.hash(newPassword, 10);

        const user = await partnerotpSchema.findOneAndUpdate({ email }, { password: hashedPassword });

        if (!user) {
            return res.status(404).json({ error: "User not found" });
        }

        await sendPasswordEmail(email, newPassword);

        return res.json({ message: "Password updated successfully. Your new password has been sent to your email." });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
};

// // 🔹 Update Password Function
// export const setPassword = async (req, res) => {
//     try {
//         console.log("Request Body:", req.body); // 🔍 Debugging Line

//         const { newPassword, confirmPassword } = req.body;

//         if (!newPassword || !confirmPassword) {
//             return res.status(400).json({ error: "Both password fields are required" });
//         }

//         if (newPassword !== confirmPassword) {
//             return res.status(400).json({ error: "Passwords do not match" });
//         }

//         const email = req.user.email;
//         const hashedPassword = await bcrypt.hash(newPassword, 10);

//         const user = await User.findOneAndUpdate({ email }, { password: hashedPassword });

//         if (!user) {
//             return res.status(404).json({ error: "User not found" });
//         }

//         await sendPasswordEmail(email, newPassword);

//         return res.json({ message: "Password updated successfully. Your new password has been sent to your email." });
//     } catch (error) {
//         res.status(500).json({ error: error.message });
//     }
// };

export const signIn = async (req, res) => {
    try {
        const { email, password } = req.body;

        const user = await User.findOne({ email });
        if (!user) {
            return res.status(404).json({ error: "User not found" });
        }

        const isPasswordValid = await bcrypt.compare(password, user.password);
        if (!isPasswordValid) {
            return res.status(401).json({ error: "Invalid password" });
        }

        const token = generateToken(user);
        res.json({ message: "Login successful", token });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
};







// const verifyRecaptcha = async (recaptchaToken) => {
//     try {
//         const response = await axios.post(
//             `https://www.google.com/recaptcha/api/siteverify?secret=${process.env.RECAPTCHA_SECRET_KEY}&response=${recaptchaToken}`
//         );

//         return response.data.success;
//     } catch (error) {
//         console.error("reCAPTCHA Verification Failed:", error.message);
//         return false;
//     }
// };







// Function to send reset email
const sendResetPasswordEmail = async (email, resetToken) => {
    try {
        if (!process.env.EMAIL_USER || !process.env.EMAIL_PASS || !process.env.FRONTEND_URL) {
            throw new Error("Missing required environment variables for email sending.");
        }

        const resetLink = `${process.env.FRONTEND_URL}/reset-password?token=${resetToken}`;

        const transporter = nodemailer.createTransport({
            service: "gmail",
            auth: {
                user: process.env.EMAIL_USER,
                pass: process.env.EMAIL_PASS,
            },
        });

        const mailOptions = {
            from: `"Support Team" <${process.env.EMAIL_USER}>`,
            to: email,
            subject: "Reset Your Password",
            html: `<div style="font-family: Arial, sans-serif; max-width: 500px; margin: auto; padding: 20px; border: 1px solid #ddd; border-radius: 8px;">
                    <h2 style="color: #D93025; text-align: center;">Password Reset Request</h2>
                    <p style="font-size: 16px; text-align: center;">You requested to reset your password. Click the button below to reset it:</p>
                    <div style="text-align: center; margin: 20px;">
                        <a href="${resetLink}" style="background: #4285F4; color: #fff; padding: 12px 20px; text-decoration: none; font-size: 18px; border-radius: 5px;">Reset Password</a>
                    </div>
                    <p style="font-size: 14px; text-align: center;">If you did not request this, please ignore this email.</p>
                </div>`,
        };

        await transporter.sendMail(mailOptions);
        console.log("Reset password email sent to:", email);
    } catch (error) {
        console.error("Error sending reset email:", error);
        throw new Error("Failed to send reset email.");
    }
};

// Forgot Password Function
export const forgotPassword = async (req, res) => {
    try {
        const { email } = req.body;
        if (!email) return res.status(400).json({ message: "Email is required" });

        const user = await User.findOne({ email });
        if (!user) return res.status(404).json({ message: "User not found" });

        // Generate secure reset token
        const resetToken = crypto.randomBytes(32).toString("hex");
        const hashedToken = crypto.createHash("sha256").update(resetToken).digest("hex");

        console.log("Generated Token:", resetToken);
        console.log("Hashed Token Before Saving:", hashedToken);

        // Store hashed token and expiration in the database
        user.resetToken = hashedToken;
        user.resetTokenExpires = Date.now() + 24 * 60 * 60 * 1000; // 24 hours
        await user.save();

        // Send reset password email with the plain token
        await sendResetPasswordEmail(email, resetToken);

        res.status(200).json({ message: "Password reset link sent to your email" });

    } catch (error) {
        console.error("Forgot Password Error:", error);
        res.status(500).json({ message: "Internal server error" });
    }
};

// Reset Password Function
export const resetPassword = async (req, res) => {
    try {
        const { token, newPassword, confirmPassword } = req.body;

        // Validate inputs
        if (!token || !newPassword || !confirmPassword) {
            return res.status(400).json({ message: "Token, new password, and confirm password are required." });
        }

        if (newPassword.length < 6) {
            return res.status(400).json({ message: "Password must be at least 6 characters long." });
        }

        if (newPassword !== confirmPassword) {
            return res.status(400).json({ message: "New password and confirm password do not match." });
        }

        // Hash the received token before searching in DB
        const hashedToken = crypto.createHash("sha256").update(token).digest("hex");

        console.log("Received Token:", token);
        console.log("Hashed Token for Query:", hashedToken);

        // Find user with the hashed reset token & ensure token is valid
        const user = await User.findOne({
            resetToken: hashedToken,
            resetTokenExpires: { $gt: Date.now() } // Ensure token is still valid
        });

        if (!user) {
            console.log("User not found or token expired.");
            return res.status(400).json({ message: "Invalid or expired token." });
        }

        console.log("User Found:", user.email);
        console.log("Token Expiry Time:", user.resetTokenExpires);
        console.log("Time Difference:", user.resetTokenExpires - Date.now());

        // Hash the new password
        const hashedPassword = await bcrypt.hash(newPassword, 10);

        // Update user's password and remove reset token fields
        user.password = hashedPassword;
        user.resetToken = undefined;
        user.resetTokenExpires = undefined;

        await user.save();

        console.log("Password reset successful for:", user.email);

        return res.status(200).json({ message: "Password successfully reset. You can now log in." });

    } catch (error) {
        console.error("Reset Password Error:", error);
        return res.status(500).json({ message: "Internal server error." });
    }
};
