import jwt from "jsonwebtoken";

const authenticateToken = (req, res, next) => {
    try {
        const authHeader = req.headers["authorization"];

        if (!authHeader) {
            return res.status(401).json({ error: "Token missing" });
        }

        // Allow both "Bearer TOKEN" and direct token formats
        const token = authHeader.startsWith("Bearer ") ? authHeader.split(" ")[1] : authHeader;

        if (!token) {
            return res.status(401).json({ error: "Invalid token format" });
        }

        jwt.verify(token, process.env.JWT_SECRET, (err, decoded) => {
            if (err) {
                console.error("❌ JWT Verification Error:", err.message);

                // Differentiate between invalid and expired tokens
                if (err.name === "TokenExpiredError") {
                    return res.status(401).json({ error: "Token has expired, please log in again" });
                }
                return res.status(403).json({ error: "Invalid token" });
            }

            console.log("✅ Token Decoded:", decoded);
            console.log("🕒 Token Expiry Time:", new Date(decoded.exp * 1000)); // Convert UNIX timestamp

            req.user = decoded; // Attach decoded user info to request
            next();
        });
    } catch (error) {
        console.error("❌ Authentication Middleware Error:", error);
        return res.status(500).json({ error: "Internal server error" });
    }
};

export default authenticateToken; // ✅ Use default export
