import mongoose from "mongoose";
const ObjectId = mongoose.Types.ObjectId;

const userSchema = new mongoose.Schema({
    email: { type: String, required: true, unique: true },
    password: { type: String,  },
    city_id:{
        type:mongoose.Types.ObjectId
    },
    // OTP Fields
    otp: { type: String },
    otpExpiry: { type: Date },

    // Reset Token Fields
    resetToken: { type: String, index: true },
    resetTokenExpires: { type: Date, index: true },

    // User Type (Role Selection)
    user_type: { 
        type: String, 
        required: true, 
        enum: ["agent", "drop_location", "full_franchise"] 
    },

    // Approval Status
    is_approved: { type: Boolean, default: false },

    // Common Fields (all users)
    phone: { type: String, required: true },

    // Fields specific to Agent
    city: { type: String },
    name: { type: String },
    vehicle_type: { type: String, enum: ["Three Wheeler", "Four Wheeler"] },
    aadhar_no: { type: String },
    image: { type: String },
    aadhar_image: { type: String },
    vehicle_regd_image: { type: String },
    vehicle_insurance_image: { type: String },

    // Fields specific to Drop Location
    shop_address: { type: String },
    shop_name: { type: String },
    shop_register_number: { type: String },
    full_name: { type: String },
    reference_number: { type: String },

    // Fields specific to Franchise
    // franchiseName: { type: String },
    ownerName: { type: String },
    address: { type: String },
    referenceNumber: { type: String, unique: true },

}, { timestamps: true });

const User = mongoose.model("User", userSchema);
export default User;
